# -*- coding: UTF-8 -*-

from artix.common import py2to3utils
import logger
import os
import shutil
import tempfile

wgetErrorMessages = {
    0: 'No problems occurred',
    1: 'Generic error code',
    2: 'Parse error---for instance, when parsing command-line options, the .wgetrc or .netrc...',
    3: 'File I/O error',
    4: 'Network failure',
    5: 'SSL verification failure',
    6: 'Username/password authentication failure',
    7: 'Protocol errors',
    8: 'Server issued an error response'
}


def main(url, path, fileName, archivePath=None, isDict=False):
    log = logger.getLogger()
    tmpPath = tempfile.gettempdir()

    if fileName == '':
        fileName = url.split('/')[-1]

    # скачиваем файл, если ссылка не доступна в течение минуты переходим к следующему сообщению
    log.info('Начинаем попытку скачивания файла %s %s %s' % (url, path, fileName))

    returncode = downloadFile(url, tmpPath, fileName)

    if isDict and returncode == 2048:
        log.info("Файл по пути %s не найден, попытка найти справочник в архиве" % (os.path.join(tmpPath, fileName)))
        returncode = downloadFile(getArchivePath(url, archivePath), tmpPath, fileName)

    if returncode != 0:
        errorMessage = wgetErrorMessages.get(returncode, 'Неизвестная ошибка')
        log.error("wget вернул ошибку: \nКод ошибки: '%s\nСообщение: '%s'" % (returncode, errorMessage))
        return False, errorMessage

    if path:
        if not os.access(path, os.F_OK):  # проверка существует ли такая директория
            log.info('Создаем директорию ' + py2to3utils.ensureStr(path))
            os.makedirs(path)  # создаем директорию
        # копируем файл из временной директории в заданную
        log.info("Перемещаем полученный файл в директорию '%s'" % py2to3utils.ensureStr(path))
        if tmpPath != path:
            f = os.path.normpath(tmpPath + '/' + fileName)
            try:
                shutil.move(f, path)
            except Exception as e:
                mess = 'Не удалось переместить полученный файл. %s' % str(e)
                log.error(mess)
                return False, mess
        return True, 'Файл скачан'
    mess = 'Не указан путь сохранения файла'
    log.error(mess)
    return False, mess


def getArchivePath(url, archivePath):
    """ Возвращает url архива справочника.
    Если url передан в archivePath, то возвращает archivePath, иначе в url заменяет current на archive
    Keyword arguments:
    url -- url по которому находится справочник
    archivePath -- url по которому находится архив. Параметр не обязательный
    """
    return archivePath if archivePath else 'archive'.join(url.rsplit('current', 1))


def downloadFile(url, tmpPath, fileName):
    """ Скачивает файл по url в указанный каталог. Возвращает статус выхода.
    Keyword arguments:
    url -- url по которому находится файл
    tmpPath -- каталог для сохранения файла
    fileName -- имя с которым будет сохранен скаченный файл
    """
    return os.system("wget -O%s -c -t 5 -w 12 -T 60 --progress=dot:mega %s "
                     "> /dev/null 2>&1" % (os.path.join(tmpPath, fileName), url))
