# -*- coding: UTF-8 -*-
#
# Получение справочников из очереди
#

from artix.common import py2to3utils
from plugins import artixinterchange
from plugins import backupFile
from plugins import downloadfile
from plugins import status
from plugins import writetofile
from propertyReader import properties
import bz2
import logger
import os
import tempfile
import traceback

log = logger.getLogger()


def loadDict(dictPath, dictProperties):
    dictPath = os.path.normpath(dictPath) if dictPath else None
    try:
        res, mes = artixinterchange.upload(dictPath, dictProperties)
        if dictPath:
            backupFile.archive(dictPath, 'dicts')
        return res, mes
    except Exception as e:
        log.error(str(e))
        log.error(traceback.format_exc())
        status.sendDictState(dictProperties['dictCashLoadStateId'], str(e), 'Ошибка: %s' % str(e), status.STATUS_ERROR,
                properties.httpHostCS, properties.httpPortCS, dictProperties['version'])
        return status.errorProgramm, str(e)


def str2bool(v):
    return str(v).lower() in ("yes", "true", "t", "1")


def main(message):
    path = tempfile.gettempdir()
    fileName = 'pos.aif'
    messageId = message.id
    version = message.properties.get('version')
    dictCashLoadStateId = message.properties.get('dictCashLoadStateId' + properties.cashCode)
    if version >= '3':
        dictCashLoadStateId = message.properties.get('dictGenerateStateId')

    remoteLoadDictCommand = str2bool(message.properties.get('remoteLoadDictCommand', 'False'))
    nesGenerateCommandMode = message.properties.get('nesGenerateCommandMode', 'increment')  # ['increment', 'full', 'none']
    nesGenerateCommandCashesList = message.properties.get('nesGenerateCommandCashesList', '')
    unloadElementName = message.properties.get('unloadElementName', None)
    nesGenerateCommandOwner = message.properties.get('nesGenerateCommandOwner', '')
    
    dictProperties = {
        'messageId': messageId,
        'version': version,
        'dictCashLoadStateId': dictCashLoadStateId,
        'nesGenerateCommandMode': nesGenerateCommandMode,
        'nesGenerateCommandCashesList': nesGenerateCommandCashesList,
        'unloadElementName': unloadElementName,
        'remoteLoadDictCommand': remoteLoadDictCommand,
        'nesGenerateCommandOwner': nesGenerateCommandOwner
    }
    
    status.sendDictState(dictCashLoadStateId, 'Get dict from queue', 'Получаем справочник из очереди',
            status.STATUS_DURING, properties.httpHostCS, properties.httpPortCS, version)

    log.info('Определяем способ передачи справочника')
    dictLocation = message.properties.get('dictLocation')
    if dictLocation == 'body':
        log.info('Передача справочника через тело сообщения. Получаем тело сообщения')
        body = message.content
        if body or remoteLoadDictCommand:
            if body:
                writeFileName = os.path.splitext(fileName)[0] + '.bz2'
                writetofile.main(body, path, writeFileName, 'wb')
                decompressFile(os.path.join(path, writeFileName), os.path.join(path, fileName))

                # Загрузка справочника в указанном формате
                return loadDict(os.path.join(path, fileName), dictProperties)
            return loadDict(None, dictProperties)
        else:
            mes = 'Cправочник не указан в теле сообщения'
            log.warning(mes)
            return status.errorLoadDict, mes
    elif dictLocation == 'url':
        log.info('Передача через URL. Получаем URL справочника')
        url = py2to3utils.ensureStr(message.content)
        if url or remoteLoadDictCommand:
            if url:
                url = url.replace('{host}', str(properties.dictUrlHostCS)).replace('{port}', str(properties.dictUrlPortCS))
                log.info('URL - %s' % url)
                archivePath = message.properties.get('archivePath')
                if archivePath:
                    archivePath = archivePath.replace('{host}', str(properties.dictUrlHostCS)).replace('{port}', str(properties.dictUrlPortCS))
                fileName = url.split('/')[-1]
                res, mes = downloadfile.main(url, path, fileName, archivePath, True)
                if res:
                    newFileName = fileName.replace('.bz2', '')
                    decompressFile(os.path.join(path, fileName), os.path.join(path, newFileName))
                    fileName = newFileName

                # Загрузка справочника в указанном формате
                if res:
                    res, mes = loadDict(os.path.join(path, fileName), dictProperties)
                else:
                    res = status.errorLoadDict
            else:
                res, mes = loadDict(None, dictProperties)
            return res, mes
        else:
            mes = 'Не указана ссылка на скачивание справочника'
            log.warning(mes)
            return status.errorLoadDict, mes

    mes = 'Способ передачи справочника не определен'
    log.warning(mes)
    return status.errorLoadDict, mes


def decompressFile(sourceFile, destFile):
    data = bz2.BZ2File(sourceFile, 'rb')
    chunk = data.read(1048576)  # считываем 1 МБ
    with open(destFile, 'wb') as f:
        while chunk:
            f.write(chunk)
            chunk = data.read(1048576)  # считываем 1 Мб
    data.close()
    os.remove(sourceFile)
